@@ -32,6 +32,7 @@ Contents:
 #include "mdi_mpi.h"
 #include "mdi_lib.h"
 #include "physconst.h"
+#include "elements.h"
 
 /*! \brief MDI major version number */
 const int MDI_MAJOR_VERSION = MDI_MAJOR_VERSION_;
@@ -777,6 +778,31 @@ int MDI_Conversion_factor(const char* in_unit, const char* out_unit, double* con
   return 0;
 }
 
+/* \brief Convert an element name to its atomic number */
+int MDI_String_to_atomic_number(const char* element_symbol, int* atomic_number)
+{
+  int i;
+  int found_number = -1;
+
+  for (i=0; i<118; i++){
+    if ( strcmp(element_symbol, elements[i]) == 0 ) {
+
+      // The atomic number is 1 + the index in the list.
+      found_number = i + 1;
+      break;
+    };
+  };
+
+  if ( found_number == -1 ) {
+    mdi_error("Unrecognized element name, unable to look up atomic number.");
+    return 1;
+  }
+  else {
+    *atomic_number = found_number;
+    return 0;
+  };
+
+}
 
 /*! \brief Get the role of the code
  *
@@ -984,7 +1010,7 @@ int MDI_Check_node_exists(const char* node_name, MDI_Comm comm, int* flag)
   }
 
   // confirm that the node_name size is not greater than MDI_COMMAND_LENGTH
-  if ( strlen(node_name) > COMMAND_LENGTH ) {
+  if ( strlen(node_name) > MDI_COMMAND_LENGTH_ ) {
     mdi_error("Node name is greater than MDI_COMMAND_LENGTH");
     return 2;
   }
@@ -1190,13 +1216,13 @@ int MDI_Check_command_exists(const char* node_name, const char* command_name, MD
   }
 
   // confirm that the node_name size is not greater than MDI_COMMAND_LENGTH
-  if ( strlen(node_name) > COMMAND_LENGTH ) {
+  if ( strlen(node_name) > MDI_COMMAND_LENGTH_ ) {
     mdi_error("Node name is greater than MDI_COMMAND_LENGTH");
     return 2;
   }
 
   // confirm that the command_name size is not greater than MDI_COMMAND_LENGTH
-  if ( strlen(command_name) > COMMAND_LENGTH ) {
+  if ( strlen(command_name) > MDI_COMMAND_LENGTH_ ) {
     mdi_error("Cannot chcek command name with length greater than MDI_COMMAND_LENGTH");
     return 3;
   }
@@ -1269,7 +1295,7 @@ int MDI_Get_ncommands(const char* node_name, MDI_Comm comm, int* ncommands)
   }
 
   // confirm that the node_name size is not greater than MDI_COMMAND_LENGTH
-  if ( strlen(node_name) > COMMAND_LENGTH ) {
+  if ( strlen(node_name) > MDI_COMMAND_LENGTH_ ) {
     mdi_error("Node name is greater than MDI_COMMAND_LENGTH");
     return 2;
   }
@@ -1440,13 +1466,13 @@ int MDI_Check_callback_exists(const char* node_name, const char* callback_name,
   }
 
   // confirm that the node_name size is not greater than MDI_COMMAND_LENGTH
-  if ( strlen(node_name) > COMMAND_LENGTH ) {
+  if ( strlen(node_name) > MDI_COMMAND_LENGTH_ ) {
     mdi_error("Node name is greater than MDI_COMMAND_LENGTH");
     return 2;
   }
 
   // confirm that the callback_name size is not greater than MDI_COMMAND_LENGTH
-  if ( strlen(callback_name) > COMMAND_LENGTH ) {
+  if ( strlen(callback_name) > MDI_COMMAND_LENGTH_ ) {
     mdi_error("Cannot check callback name with length greater than MDI_COMMAND_LENGTH");
     return 3;
   }
@@ -1519,7 +1545,7 @@ int MDI_Get_ncallbacks(const char* node_name, MDI_Comm comm, int* ncallbacks)
   }
 
   // confirm that the node_name size is not greater than MDI_COMMAND_LENGTH
-  if ( strlen(node_name) > COMMAND_LENGTH ) {
+  if ( strlen(node_name) > MDI_COMMAND_LENGTH_ ) {
     mdi_error("Node name is greater than MDI_COMMAND_LENGTH");
     return 2;
   }
@@ -1707,6 +1733,56 @@ int MDI_Launch_plugin(const char* plugin_name, const char* options, void* mpi_co
 }
 
 
+/*! \brief Open an MDI plugin instance in the background
+ *
+ * The function returns \p 0 on a success.
+ *
+ * \param [in]       plugin_name
+ *                   Name of the plugin.
+ * \param [in]       options
+ *                   Command-line options for the plugin.
+ * \param [in]       mpi_comm_ptr
+ *                   Pointer to an MPI intra-communicator that spans all ranks that will run this plugin instance.
+ * \param [out]      mdi_comm_ptr
+ *                   Pointer to an MDI communicator for communication with the launched plugin.
+ */
+int MDI_Open_plugin(const char* plugin_name, const char* options, void* mpi_comm_ptr,
+                       MDI_Comm* mdi_comm_ptr) {
+#if _MDI_PLUGIN_SUPPORT == 1
+  int ret = library_open_plugin(plugin_name, options, mpi_comm_ptr,
+                                   mdi_comm_ptr);
+  return ret;
+#else
+  mdi_error("MDI_Ilaunch_plugin was called, but this build of the MDI Library was built without plugin support.");
+  return 1;
+#endif
+}
+
+
+/*! \brief Close an MDI plugin instance
+ *
+ * The function returns \p 0 on a success.
+ *
+ * \param [in]       plugin_name
+ *                   Name of the plugin.
+ * \param [in]       options
+ *                   Command-line options for the plugin.
+ * \param [in]       mpi_comm_ptr
+ *                   Pointer to an MPI intra-communicator that spans all ranks that will run this plugin instance.
+ * \param [out]      mdi_comm_ptr
+ *                   Pointer to an MDI communicator for communication with the launched plugin.
+ */
+int MDI_Close_plugin(MDI_Comm mdi_comm) {
+#if _MDI_PLUGIN_SUPPORT == 1
+  int ret = library_close_plugin(mdi_comm);
+  return ret;
+#else
+  mdi_error("MDI_Close_plugin was called, but this build of the MDI Library was built without plugin support.");
+  return 1;
+#endif
+}
+
+
 /*! \brief Set the callback MDI uses for MDI_Execute_Command
  *
  * The function returns \p 0 on a success.
@@ -1847,6 +1923,19 @@ int MDI_Set_Mpi4py_Send_Callback(int (*mpi4py_send)(void*, int, int, int, MDI_Co
 }
 
 
+/*! \brief Set the callback MDI uses for gathering MDI versions when using mpi4py
+ *
+ * The function returns \p 0 on a success.
+ *
+ * \param [in]       mpi4py_allgather
+ *                   Function pointer to the mpi4py_allgather callback
+ */
+int MDI_Set_Mpi4py_Allgather_Callback(int (*mpi4py_allgather)(void*, void*)) {
+  mpi4py_allgather_callback = mpi4py_allgather;
+  return 0;
+}
+
+
 /*! \brief Set the callback MDI uses for gathering code names when using mpi4py
  *
  * The function returns \p 0 on a success.
@@ -1854,7 +1943,7 @@ int MDI_Set_Mpi4py_Send_Callback(int (*mpi4py_send)(void*, int, int, int, MDI_Co
  * \param [in]       mpi4py_gather_names
  *                   Function pointer to the mpi4py_gather_names callback
  */
-int MDI_Set_Mpi4py_Gather_Names_Callback(int (*mpi4py_gather_names)(void*, void*)) {
+int MDI_Set_Mpi4py_Gather_Names_Callback(int (*mpi4py_gather_names)(void*, void*, int*, int*)) {
   mpi4py_gather_names_callback = mpi4py_gather_names;
   return 0;
 }
